//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                           DateTimeInstance.mqh |
//|                                                                                                                                                                            hoz |
//|                                                                                                                                                                                |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
#property copyright "hoz"
#property link      ""

//+================================================================================================================================================================================+
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//|                                                              СТРУКТУРА-ЭКЗЕМПЛЯР, ПРЕДСТАВЛЯЮЩАЯ СОБОЙ ДАТУ И ВРЕМЯ                                                            |
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+================================================================================================================================================================================+
struct DateTimeInstance {
  MqlDateTime m_dtStructure;
  //---- Конструкторы
  DateTimeInstance(datetime dt);
  DateTimeInstance(datetime dt, int hour, int minute, int second);
  DateTimeInstance(const DateTimeInstance& instance);
  DateTimeInstance(const DateTimeInstance& instance, int hour, int minute, int second);

  //--- Переопределение операторов
  void operator=(DateTimeInstance& dt);
  bool operator==(DateTimeInstance& dt);
  bool operator!=(DateTimeInstance& dt);
  bool operator<(DateTimeInstance& dt);
  bool operator>(DateTimeInstance& dt);
  bool operator<=(DateTimeInstance& dt);
  bool operator>=(DateTimeInstance& dt);
};
//==================================================================================================================================================================================
// Конструкторы структуры. =========================================================================================================================================================
DateTimeInstance :: DateTimeInstance(datetime dt) {
  TimeToStruct(dt, m_dtStructure);
}

DateTimeInstance :: DateTimeInstance(datetime dt,     // Значение типа datetime
                                     int hour,        // Количество часов
                                     int minute,      // Количество минут
                                     int second) {    // Количество секунд
  TimeToStruct(dt, m_dtStructure);
  m_dtStructure.hour = hour;
  m_dtStructure.min = minute;
  m_dtStructure.sec = second;
}

DateTimeInstance :: DateTimeInstance(const DateTimeInstance& instance) {
  m_dtStructure.year = instance.m_dtStructure.year;
  m_dtStructure.mon = instance.m_dtStructure.mon;
  m_dtStructure.day = instance.m_dtStructure.day;
  m_dtStructure.hour = instance.m_dtStructure.hour;
  m_dtStructure.min = instance.m_dtStructure.min;
  m_dtStructure.sec = instance.m_dtStructure.sec;
}

DateTimeInstance :: DateTimeInstance(const DateTimeInstance& instance, int hour, int minute, int second) {
  m_dtStructure.year = instance.m_dtStructure.year;
  m_dtStructure.mon = instance.m_dtStructure.mon;
  m_dtStructure.day = instance.m_dtStructure.day;
  m_dtStructure.hour = instance.m_dtStructure.hour + hour;
  m_dtStructure.min = instance.m_dtStructure.min + minute;
  m_dtStructure.sec = instance.m_dtStructure.sec + second;
}
//==================================================================================================================================================================================
// Переопределение оператора `=`. ==================================================================================================================================================
void DateTimeInstance :: operator=(DateTimeInstance& dt) {
//---
  m_dtStructure.year = dt.m_dtStructure.year;
  m_dtStructure.mon = dt.m_dtStructure.mon;
  m_dtStructure.day = dt.m_dtStructure.day;
  m_dtStructure.hour = dt.m_dtStructure.hour;
  m_dtStructure.min = dt.m_dtStructure.min;
  m_dtStructure.sec = dt.m_dtStructure.sec;
}
//==================================================================================================================================================================================
// Переопределение оператора `==`. =================================================================================================================================================
bool DateTimeInstance :: operator==(DateTimeInstance& dt) {
//---
  if (m_dtStructure.year != dt.m_dtStructure.year)
    return false;
  if (m_dtStructure.mon != dt.m_dtStructure.mon)
    return false;
  if (m_dtStructure.day != dt.m_dtStructure.day)
    return false;
  if (m_dtStructure.hour != dt.m_dtStructure.hour)
    return false;
  if (m_dtStructure.min != dt.m_dtStructure.min)
    return false;
  if (m_dtStructure.sec != dt.m_dtStructure.sec)
    return false;
//---
  return true;
}
//==================================================================================================================================================================================
// Переопределение оператора `!=`. =================================================================================================================================================
bool DateTimeInstance :: operator!=(DateTimeInstance& dt) {
//---
  if (m_dtStructure.year != dt.m_dtStructure.year)
    return true;
  if (m_dtStructure.mon != dt.m_dtStructure.mon)
    return true;
  if (m_dtStructure.day != dt.m_dtStructure.day)
    return true;
  if (m_dtStructure.hour != dt.m_dtStructure.hour)
    return true;
  if (m_dtStructure.min != dt.m_dtStructure.min)
    return true;
  if (m_dtStructure.sec != dt.m_dtStructure.sec)
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `<`. ==================================================================================================================================================
bool DateTimeInstance :: operator<(DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) < StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `>`. ==================================================================================================================================================
bool DateTimeInstance :: operator>(DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) > StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `<=`. ==================================================================================================================================================
bool DateTimeInstance :: operator<=(DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) <= StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `>`. ==================================================================================================================================================
bool DateTimeInstance :: operator>=(DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) >= StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
